(ns document-storage.git.repository
  (:require [clj-jgit.porcelain :as jp]
            [clojure.tools.logging :as log]
            [clojure.java.io :as io]))


(defonce repositories (atom {}))
(defonce default-repository :default)

(defn init-repository [dir]
  (when-not (jp/discover-repo dir)
    (log/info dir " is not a Git-Repository")
    (log/info "Initialize Git-Repository in " dir)
    (jp/git-init :dir dir))
  (jp/load-repo (jp/discover-repo dir)))



(defn get-file-path [id repository]
  (let [repo-directory (get-in @repositories [repository :directory])
        file-path (str repo-directory "/" id)]
    file-path))

(defn cached-repository? [repository]
  (get-in @repositories [repository :caching]))

(defn no-commit-repository? [repository]
  (get-in @repositories [repository :no-commit]))

(defn get-repo-data [id]
  (if-let [repo (get-in @repositories [id])]
    repo
    (log/error "invalid repository " id)
    ))

(defn get-repo
  ([]
   (get-repo default-repository))
  ([id]
   (:git-repo (get-repo-data id))))


(defn get-repository-path [repository]
  (let [repo-directory (get-in @repositories [repository :directory])]
    repo-directory))

(defn valid-id? [id repository]
  (let [file-path (get-file-path id repository)]
    (.exists (io/as-file file-path))))

