(ns document-storage.git.core-test
  (:require [clojure.test :refer :all]
            [document-storage.core :as ds]
            [document-storage.git.views :as dsv]
            [document-storage.git.core :as gstorage]
            [clojure.java.io :as io]
            [fipp.edn :refer [pprint] :rename {pprint fipp}]
            [clj-helper.string :refer [get-unique-id]]
            [clojure.data :refer [diff]]
            [document-storage.git.core :as dsgit]
            [document-storage.protocol :as dsprot]
            [clojure.tools.logging :as log]
            [document-storage.protocol-test :as protocol-tests]))

(def ds-atom (atom nil))

(defn create-storage [directory]
  (gstorage/create-storage directory {:repositories
                                      {:testrepo {}}}))


(defn delete-folder
  [f1 & [silently]]
  (when (.isDirectory (io/file f1))
    (doseq [f2 (.listFiles (io/file f1))]
      (delete-folder f2 silently)))
  (io/delete-file f1 silently))

(defn destroy-storage [directory]
  (delete-folder directory))

(defn storage-fixture [f]
  (create-storage "teststorage")
  (reset! ds-atom (ds/make-git-ds "teststorage"))
  (f)
  (destroy-storage "teststorage"))

(use-fixtures :once storage-fixture)

(deftest simple-test
  (protocol-tests/test-save-document ds-atom))


(deftest versioning-test
  (protocol-tests/test-versioning ds-atom))

(deftest find-document-test
  (protocol-tests/test-find-documents ds-atom))

(deftest views-test
  (testing "save views, save basic structure, load it and load views"
    (let [structure {:a 5 :c [1 2 3 4 99 5 -4] :b 88}
          id "simple-test1"
          repo :testrepo
          fn1 (fn [d]
                {:sum1 (+ (:a d)
                          (:b d))
                 :sum2 (reduce + (:c d))})
          fn2 (fn [d]
                {:sum1 (- (:a d)
                          (:b d))
                 :c-copy (:c d)})]
      (dsv/add-view repo :fn1 fn1)
      (dsv/add-view repo :fn2 fn2)
      (.save-document @ds-atom id structure repo)
      (is (= structure
             (.load-document @ds-atom id repo)))
      (let [views (dsv/load-views id :repository repo)
            v1 (:fn1 views)
            v2 (:fn2 views)]
        (is (= (:sum1 v1) 93))
        (is (= (:sum2 v1) 110))
        (is (= (:sum1 v2) -83))))))
