(ns document-storage.testdata
  (:require  [clojure.test :as t]
             [clojure.spec.alpha :as s]
             [clojure.spec.gen.alpha :as gen]
             [clojure.string :as str]))


(s/def :content-element.html/content (s/and
                                      string?
                                      #(> (count %) 32)))
(s/def :content-element.text/content (s/and
                                      string?
                                      #(> (count %) 32)))


(s/def :content-element/id string?)
(s/def :content-element/type #{:html :text})

(defmulti content-type :type)
(defmethod content-type :html [_]
  (s/keys :req-un [:content-element/id :content-element/type :content-element.html/content]))

(defmethod content-type :text [_]
  (s/keys :req-un [:content-element/id :content-element/type :content-element.text/content]))


(s/def ::content-element (s/multi-spec content-type :type))

(s/explain-str ::content-element {:type :html})

(s/def :document/content (s/coll-of ::content-element
                                    :into []
                                    :gen-max 35))


(s/def :document/created-at inst?)

(s/def :document/updated-at (s/nilable inst?))

(s/def :document/title (s/and
                        string?
                        #(> (count %) 10)))

(s/def :document/tags (s/coll-of string?
                                 :into #{}
                                 :gen-max 6))

(s/def ::document (s/keys :req-un [:document/title
                                   :document/tags
                                   :document/created-at
                                   :document/updated-at
                                   :document/content]))

(defn generate-document []
  (gen/generate (s/gen ::document)))

